@@ -41,11 +41,13 @@ namespace Beagle.Util {
 
 		public const string UnindexedNamespace = "_unindexed:";
 
-		private const String timeFormat = "yyyyMMddHHmmss";
+		private const String TimeFormat = "yyyyMMddHHmmss";
+		// FIXME: Fix all the UTC and timezone hack when switching to .Net-2.0
+		private const String LocalTimeFormat = "yyyyMMddHHmmsszz";
 
 		static public string DateTimeToString (DateTime dt)
 		{
-			return dt.ToString (timeFormat);
+			return dt.ToString (TimeFormat);
 		}
 
 		static public string DateTimeToYearMonthString (DateTime dt)
@@ -63,7 +65,14 @@ namespace Beagle.Util {
 			if (str == null || str == "")
 				return new DateTime ();
 
-			return DateTime.ParseExact (str, timeFormat, CultureInfo.CurrentCulture);
+			str = string.Concat (str, "+00");
+			// Uncomment next 3 lines to see what how 20061107173446 (which is stored in UTC)
+			// used to be parsed as 2006-11-07T17:34:46.0000000-05:00
+			//DateTime dt = DateTime.ParseExact (str, LocalTimeFormat, CultureInfo.InvariantCulture);
+			//Console.WriteLine ("Parsed {0} as {1},{2}", str, dt, dt.ToString("yyyy-MM-ddTHH:mm:ss.fffffffzzz", CultureInfo.InvariantCulture));
+			//return dt;
+			// If no timezone is present, parse_exact uses local time zone
+			return DateTime.ParseExact (str, LocalTimeFormat, CultureInfo.InvariantCulture);
                 }
 
 		static public string DateTimeToFuzzy (DateTime dt)
@@ -147,6 +156,24 @@ namespace Beagle.Util {
 			
 			return span_str;
 		}
+
+		static public string TimeSpanToString (TimeSpan span)
+		{
+			StringBuilder sb = new StringBuilder ();
+
+			if (span.Days > 0)
+				sb.Append (span.Days + "d");
+
+			if (span.TotalHours > 1.0)
+				sb.Append (span.Hours + "h");
+
+			if (span.TotalMinutes > 1.0)
+				sb.Append (span.Minutes + "m");
+
+			sb.Append (span.Seconds + "s");
+
+			return sb.ToString ();
+		}
 		
 		static public string FileLengthToString (long len)
 		{
@@ -305,7 +332,7 @@ namespace Beagle.Util {
 
 			foreach (char c in str) {
 
-				if (Array.IndexOf (CharsToQuote, c) != -1)
+				if (ArrayFu.IndexOfChar (CharsToQuote, c) != -1)
 					builder.Append (Uri.HexEscape (c));
 				else if (c < 128)
 					builder.Append (c);
