@@ -56,9 +56,16 @@ namespace Beagle.Daemon {
 				// the attributes, clobber the old attributes and the old unique Guid.
 				if (attr == null
 				    || (unique_id != Guid.Empty && unique_id != attr.UniqueId)) {
+					// First drop the old attribute, if there is one.
+					if (attr != null)
+						ifas.Drop (path);
+
+					// Now create the new attribute
 					attr = new FileAttributes ();
 					attr.UniqueId = unique_id;
 					attr.Path = path;
+					
+					// Now add the new attribute
 					ifas.Write (attr);
 					created = true;
 				}
@@ -107,6 +114,15 @@ namespace Beagle.Daemon {
 
 		//////////////////////////////////////////////////////////
 
+		public static bool IsUpToDate (string path, FileAttributes attr)
+		{
+			if (attr == null)
+				return false;
+
+			// FIXME:.Net-2.0 DateTime - Compare attr.LastWriteTime without converting to UTC
+			return (attr.LastWriteTime.ToUniversalTime () >= FileSystem.GetLastWriteTimeUtc (path));
+		}
+
 		public bool IsUpToDate (string path, Filter filter)
 		{
 			FileAttributes attr;
@@ -137,15 +153,12 @@ namespace Beagle.Daemon {
 					return false;
 			} 
 
-			if (FileSystem.GetLastWriteTimeUtc (path) > attr.LastWriteTime)
-				return false;
-
-			return true;
+			return IsUpToDate (path, attr);
 		}
 
 		public bool IsUpToDate (string path)
 		{
-			return IsUpToDate (path, null);
+			return IsUpToDate (path, (Filter) null);
 		}
 
 		//////////////////////////////////////////////////////////
