@@ -76,12 +76,13 @@ namespace Beagle.Daemon {
 				}
 			}
 
-			foreach (DictionaryEntry entry in matched_filters_by_flavor) {
-				FilterFlavor flav = (FilterFlavor) entry.Key;
-				Filter filter = (Filter) entry.Value;
-				
-				if (Debug)
+			if (Debug) {
+				foreach (DictionaryEntry entry in matched_filters_by_flavor) {
+					FilterFlavor flav = (FilterFlavor) entry.Key;
+					Filter filter = (Filter) entry.Value;
+					
 					Logger.Log.Debug ("Found matching filter: {0}, Weight: {1}", filter, flav.Weight);
+				}
 			}
 
 			return matched_filters_by_flavor.Values;
@@ -133,42 +134,6 @@ namespace Beagle.Daemon {
 
 		/////////////////////////////////////////////////////////////////////////
 
-		static public TextReader FilterFile (string path)
-		{
-			return FilterFile (path, null);
-		}
-		
-		static public TextReader FilterFile (string path, string mime_type)
-		{
-			if (mime_type == null)
-				mime_type = XdgMime.GetMimeType (path);
-
-			if (mime_type == null)
-				return null;
-
-			ICollection filters = CreateFilters (UriFu.PathToFileUri (path), Path.GetExtension (path), mime_type);
-			TextReader reader;
-			
-			foreach (Filter candidate_filter in filters) {
-				if (Debug)
-					Logger.Log.Debug ("Testing filter: {0}", candidate_filter);
-						
-				// Open the filter, and hook up the TextReader.
-				if (candidate_filter.Open (path)) {
-					reader = candidate_filter.GetTextReader ();
-					
-					if (Debug)
-						Logger.Log.Debug ("Successfully filtered {0} with {1}", path, candidate_filter);
-					
-					return reader;
-				} else if (Debug) {
-					Logger.Log.Debug ("Unsuccessfully filtered {0} with {1}, falling back", path, candidate_filter);
-				}
-			}		
-
-			return null;
-		} 
-
 		static private bool ShouldWeFilterThis (Indexable indexable)
 		{
 			if (indexable.Filtering == IndexableFiltering.Never
@@ -222,11 +187,11 @@ namespace Beagle.Daemon {
 				if (Directory.Exists (path)) {
 					indexable.MimeType = "inode/directory";
 					indexable.NoContent = true;
-					if (! indexable.ValidTimestamp)
+					if (! indexable.ValidTimestamp && indexable.IsNonTransient)
 						indexable.Timestamp = Directory.GetLastWriteTimeUtc (path);
 				} else if (File.Exists (path)) {
 					// Set the timestamp to the best possible estimate (if no timestamp was set by the backend)
-					if (! indexable.ValidTimestamp)
+					if (! indexable.ValidTimestamp && indexable.IsNonTransient)
 						indexable.Timestamp = File.GetLastWriteTimeUtc (path);
 				} else {
 					Logger.Log.Warn ("No such file: {0}", path);
@@ -239,7 +204,7 @@ namespace Beagle.Daemon {
 				if (! indexable.NoContent) {
 					indexable.NoContent = true;
 
-					Logger.Log.Debug ("No filter for {0} ({1})", path != null ? path : indexable.Uri.ToString (), indexable.MimeType);
+					Logger.Log.Debug ("No filter for {0} ({1}) [{2}]", indexable.DisplayUri, path, indexable.MimeType);
 					return false;
 				}
 				
@@ -263,8 +228,9 @@ namespace Beagle.Daemon {
 				if (indexable.Crawled)
 					candidate_filter.EnableCrawlMode ();
 				
-				// Set the filter's URI
+				// Set the filter's URIs
 				candidate_filter.Uri = indexable.Uri;
+				candidate_filter.DisplayUri = indexable.DisplayUri;
 
 				// allow the filter access to the indexable's properties
 				candidate_filter.IndexableProperties = indexable.Properties;
@@ -284,6 +250,8 @@ namespace Beagle.Daemon {
 					succesful_open = candidate_filter.Open (binary_stream);
 					
 				if (succesful_open) {
+					if (candidate_filter.Timestamp != DateTime.MinValue)
+						indexable.Timestamp = candidate_filter.Timestamp;
 					foreach (Property prop in candidate_filter.Properties)
 						indexable.AddProperty (prop);
 					indexable.SetTextReader (candidate_filter.GetTextReader ());
@@ -327,14 +295,13 @@ namespace Beagle.Daemon {
 		{
 			int count = 0;
 
-			foreach (Type t in ReflectionFu.ScanAssemblyForClass (assembly, typeof (Filter))) {
+			foreach (Type t in ReflectionFu.GetTypesFromAssemblyAttribute (assembly, typeof (FilterTypesAttribute))) {
 				Filter filter = null;
 
 				try {
 					filter = (Filter) Activator.CreateInstance (t);
 				} catch (Exception ex) {
-					Logger.Log.Error ("Caught exception while instantiating {0}", t);
-					Logger.Log.Error (ex);
+					Logger.Log.Error (ex, "Caught exception while instantiating {0}", t);
 				}
 
 				if (filter == null)
@@ -372,11 +339,11 @@ namespace Beagle.Daemon {
 		[XmlAttribute ("Uri")]
 		public string UriAsString {
 			get {
-				return UriFu.UriToSerializableString (uri);
+				return UriFu.UriToEscapedString (uri);
 			}
 
 			set {
-				uri = UriFu.UriStringToUri (value);
+				uri = UriFu.EscapedStringToUri (value);
 			}
 		}
 
