@@ -47,6 +47,10 @@ namespace Beagle.IndexHelper {
 		private static DateTime last_activity;
 		private static Server server;
 
+		// Current state with filtering.
+		public static Uri CurrentUri;
+		public static Filter CurrentFilter;
+
 		[DllImport ("libc")]
 		extern static private int unsetenv (string name);
 
@@ -111,15 +115,24 @@ namespace Beagle.IndexHelper {
 				server.Start ();
 				server_has_been_started = true;
 			} catch (InvalidOperationException ex) {
-				Logger.Log.Error (ex, "Couldn't start server:");
+				Logger.Log.Error (ex, "Couldn't start server.  Exiting immediately.");
 			}
 
 			if (server_has_been_started) {
 				// Set the IO priority to idle so we don't slow down the system
 				if (Environment.GetEnvironmentVariable ("BEAGLE_EXERCISE_THE_DOG") == null) {
-					if (! IoPriority.SetIdle ())
-						IoPriority.SetIoPriority (7);
-				}
+					IoPriority.ReduceIoPriority ();
+
+					int prio = Mono.Unix.Native.Syscall.nice (15);
+
+					if (prio < 0)
+						Log.Warn ("Unable to renice helper to +15");
+					else if (prio == 15)
+						Log.Debug ("Reniced helper to +15");
+					else
+						Log.Debug ("Helper was already niced to {0}, not renicing to +15", prio);
+				} else
+					Log.Info ("BEAGLE_EXERCISE_THE_DOG is set");
 				
 				// Start the monitor thread, which keeps an eye on memory usage and idle time.
 				ExceptionHandlingThread.Start (new ThreadStart (MemoryAndIdleMonitorWorker));
@@ -128,16 +141,20 @@ namespace Beagle.IndexHelper {
 				// if it terminates.
 				ExceptionHandlingThread.Start (new ThreadStart (DaemonMonitorWorker));
 
-				//Application.Run ();
+				// Start the main loop
 				main_loop.Run ();
 
-				// If we palced our sockets in a temp directory, try to clean it up
+				ExceptionHandlingThread.JoinAllThreads ();
+
+				// If we placed our sockets in a temp directory, try to clean it up
 				// Note: this may fail because the daemon is still running
 				if (PathFinder.GetRemoteStorageDir (false) != PathFinder.StorageDir) {
 					try {
 						Directory.Delete (PathFinder.GetRemoteStorageDir (false));
 					} catch (IOException) { }
 				}
+
+				Log.Info ("Index helper process shut down cleanly.");
 			}
 		}
 
@@ -239,6 +256,7 @@ namespace Beagle.IndexHelper {
 			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGINT, OurSignalHandler);
 			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGTERM, OurSignalHandler);
 			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGUSR1, OurSignalHandler);
+			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGUSR2, OurSignalHandler);
 
 			// Ignore SIGPIPE
 			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGPIPE, Mono.Unix.Native.Stdlib.SIG_IGN);
@@ -257,8 +275,11 @@ namespace Beagle.IndexHelper {
 			if (signal < 0)
 				return;
 
-			// Set shutdown flag to true so that other threads can stop initializing
-			if ((Mono.Unix.Native.Signum) signal != Mono.Unix.Native.Signum.SIGUSR1)
+			// SIGUSR1 and SIGUSR2 are informational signals.  For other
+			// signals that we handle, set the shutdown flag to true so
+			// that other threads can stop initializing
+			if ((Mono.Unix.Native.Signum) signal != Mono.Unix.Native.Signum.SIGUSR1 &&
+			    (Mono.Unix.Native.Signum) signal != Mono.Unix.Native.Signum.SIGUSR2)
 				Shutdown.ShutdownRequested = true;
 
 			// Do all signal handling work in the main loop and not in the signal handler.
@@ -267,16 +288,29 @@ namespace Beagle.IndexHelper {
 
 		private static void HandleSignal (int signal)
 		{
-			Logger.Log.Debug ("Handling signal {0} ({1})", signal, (Mono.Unix.Native.Signum) signal);
+			Log.Warn ("Handling signal {0} ({1})", signal, (Mono.Unix.Native.Signum) signal);
 
 			// If we get SIGUSR1, turn the debugging level up.
 			if ((Mono.Unix.Native.Signum) signal == Mono.Unix.Native.Signum.SIGUSR1) {
 				LogLevel old_level = Log.Level;
 				Log.Level = LogLevel.Debug;
 				Log.Debug ("Moving from log level {0} to Debug", old_level);
-				return;
 			}
 
+			string span = StringFu.TimeSpanToString (DateTime.Now - last_activity);
+
+			if (CurrentUri == null)
+				Log.Warn ("Filtering status ({0} ago): no document is currently being filtered.", span);
+			else if (CurrentFilter == null)
+				Log.Warn ("Filtering status ({0} ago): determining filter for {1}", span, CurrentUri);
+			else
+				Log.Warn ("Filtering status ({0} ago): filtering {1} with {2}", span, CurrentUri, CurrentFilter);
+
+			// Don't shut down on information signals (SIGUSR1 and SIGUSR2)
+			if ((Mono.Unix.Native.Signum) signal == Mono.Unix.Native.Signum.SIGUSR1 ||
+			    (Mono.Unix.Native.Signum) signal == Mono.Unix.Native.Signum.SIGUSR2)
+				return;
+
 			Logger.Log.Debug ("Initiating shutdown in response to signal.");
 			Shutdown.BeginShutdown ();
 		}
